Option Strict On
Option Explicit On 

Imports System.Math

Friend Module MathGaussJordan
    Private Const ERR_DSC_GaussJ_1 As String = "Singular Matrix-1 in GaussJ"
    Private Const ERR_DSC_GaussJ_2 As String = "Singular Matrix-2 in GaussJ"

    Public Sub GaussJ(ByVal A(,) As Double, ByVal B(,) As Double)
        Dim N As Integer = A.GetUpperBound(0)
        Dim M As Integer = B.GetUpperBound(1)
        Dim IndxC(N), IndxR(N) As Integer

        Dim Icol, Irow, iPiv(N) As Integer
        For I As Integer = 0 To N
            Dim Big As Double = 0.0#
            For J As Integer = 0 To N
                If iPiv(J) <> 1 Then

                    Dim K As Integer
                    For K = 0 To N
                        If iPiv(K) = 0 AndAlso Abs(A(J, K)) >= Big Then
                            Big = Abs(A(J, K))
                            Irow = J
                            Icol = K
                        ElseIf iPiv(K) > 1 Then
                            Throw New System.ArithmeticException(ERR_DSC_GaussJ_1)
                        End If
                    Next K
                End If
            Next J

            iPiv(Icol) += 1

            If Irow <> Icol Then
                For L As Integer = 0 To N
                    Dim Temp As Double = A(Irow, L)
                    A(Irow, L) = A(Icol, L)
                    A(Icol, L) = Temp
                Next L
                For L As Integer = 0 To M
                    Dim Temp As Double = B(Irow, L)
                    B(Irow, L) = B(Icol, L)
                    B(Icol, L) = Temp
                Next L
            End If

            IndxR(I) = Irow : IndxC(I) = Icol

            If A(Icol, Icol) = 0.0# Then Throw New System.ArithmeticException(ERR_DSC_GaussJ_2)

            Dim PivInv As Double = 1.0# / A(Icol, Icol)
            A(Icol, Icol) = 1.0#

            For L As Integer = 0 To N
                A(Icol, L) *= PivInv
            Next L
            For L As Integer = 0 To M
                B(Icol, L) *= PivInv
            Next L

            For LL As Integer = 0 To N
                If LL <> Icol Then
                    Dim Dum As Double = A(LL, Icol)
                    A(LL, Icol) = 0.0#
                    For L As Integer = 0 To N
                        A(LL, L) -= A(Icol, L) * Dum
                    Next L
                    For L As Integer = 0 To M
                        B(LL, L) -= B(Icol, L) * Dum
                    Next L
                End If
            Next LL
        Next I

        For I As Integer = N To 0 Step -1
            If IndxR(I) <> IndxC(I) Then
                For J As Integer = 0 To N
                    Dim Temp As Double = A(J, IndxR(I))
                    A(J, IndxR(I)) = A(J, IndxC(I))
                    A(J, IndxC(I)) = Temp
                Next J
            End If
        Next I
    End Sub
End Module